#!/usr/bin/perl
#
# Copyright (c) 2007 sitorix GmbH
# (sitorix GmbH, Stuttgart, https://www.hostthenet.de)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
# A copy of the GPL can be found at http://www.gnu.org/licenses/gpl.txt

use POSIX qw(setsid);
use Log::Log4perl qw(:easy);

# Initializating the Log System
Log::Log4perl->easy_init( { level   => $DEBUG,
                            file    => ">>/var/log/confixx_daemon.log" } );

# Read in the Config File
$cfg = "/etc/confixx_daemon.cfg";

if ( -r $cfg )
 {
  require $cfg;
 }
else
 {
 ERROR("Can't read $cfg");
 exit(0);
 }

# Load MySQL Module after the configuration File is used
use Mysql; 

###################
#  Main Programm  #
###################

# Name
$0 = "confixx-highspeed-daemon";

# Buffer flush
$| = 1;

# daemonise the program
&daemonise;

# Subroutine to daemonize the program
sub daemonise {
	chdir '/'                  	or die ERROR("Can't chdir to /: $!");
	defined($pid = fork)    	or die ERROR("Can't fork process: $!");
	open STDIN,  '/dev/null' 	or die ERROR("Can't find /dev/null: $!");
	open STDOUT, '>/dev/null';
	open STDERR, '>/dev/null';
	exit if $pid;
	setsid                     	or die ERROR("Can't start a new session: $!");
	umask 0;
}

# Infinity Loop and main program
INFO("Confixx Highspeed Daemon started up");


while(1) {

	
	until ( $dbh = Mysql->connect( "$mysql_host", "$mysql_db", "$mysql_user", "$mysql_pw" ) )
		{
			$errmsg = Mysql->errmsg();
		    	ERROR("Can't connect to the MySQL Server: $errmsg");
		    	sleep(60);
		}

	sub query
		{ 
			%hash = ();
			$dbh->selectdb("$mysql_db") ||
			 die ERROR("Can't change database to $mysql_db");
			$sth = $dbh->query( "SELECT * FROM $mysql_table" ) ||
			 die ERROR("Can't execute MySQL Query: SELECT * FROM $mysql_table");
			%hash = $sth->fetchhash ;
		}
		
	sub dolog
		{
			foreach(@output)
				{
					next if(!$_);
					ERROR("$_");
				}
			@output = ();
		}
		
	query();

	if ($hash{quota}==1) 
		{
			@output = ();
			INFO("Quota Update is 1, Executing $executable $args_quota");
		  	@output = `$executable $args_quota 2>&1` if -e $executable;
			dolog();
			query();
		}

	if ($hash{domains}==1) 
		{
			INFO("Domain Update is 1, Executing $executable $args_dns $args_mta $args_httpd");
  			@output = `$executable $args_dns $args_mta $args_httpd 2>&1` if -e $executable;
			dolog();
			query();
		}

	if ($hash{deluser}==1) 
		{
			INFO("DelUser Update is 1, Executing $executable $args_quota");
		  	@output = `$executable $args_quota 2>&1` if -e $executable;
			dolog();
			query();
		}

	if ($hash{delpop}==1) 
		{
			INFO("DelPOP Update is 1, Executing $executable $args_mta $args_system");
  			@output = `$executable $args_mta $args_system 2>&1` if -e $executable;
			dolog();
			query();
		}

	if ($hash{delftp}==1) 
		{
			INFO("DelFTP Update is 1, Executing $executable $args_system");
	  		@output = `$executable $args_system 2>&1` if -e $executable;
			dolog();
			query();
		}

	if ($hash{adduser}==1) 
		{
			INFO("AddUser Update is 1, Executing $executable $args_quota");
	  		@output = `$executable $args_quota 2>&1` if -e $executable;
			dolog();
			query();
		}

	if ($hash{addpop}==1) 
		{
			INFO("AddPOP Update is 1, Executing $executable $args_mta $args_system");
		  	@output = `$executable $args_mta $args_system 2>&1` if -e $executable;
			dolog();
			query();
		}

	if ($hash{addftp}==1) 
		{
			INFO("AddFTP Update is 1, Executing $executable $args_system");
	  		@output = `$executable $args_system 2>&1` if -e $executable;
			dolog();
			query();
		}

	if ($hash{newpwd}==1) 
		{
			INFO("NewPWD Update is 1, Executing $executable $args_system");
  			@output = `$executable $args_system 2>&1` if -e $executable;
			dolog();
			query();
		}

	if ($hash{aliases}==1) 
		{
			INFO("Alias Update is 1, Executing $executable $args_mta");
	  		@output = `$executable $args_mta 2>&1` if -e $executable;
			dolog();
			query();
		}

	$sth->finish();
	sleep(1);

}


